<?php
/**
 * @version		$Id: locallib.php,v 1.8 2010-08-06 11:05:29 juanca Exp $
 * @package		vpl. Common functions of VPL
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

define('VPL','vpl');
define('VPL_SUBMISSIONS','vpl_submissions');
define('VPL_VARIATIONS','vpl_variations');
define('VPL_ASSIGNED_VARIATIONS','vpl_assigned_variations');
define('VPL_GRADE_CAPABILITY','mod/vpl:grade');
define('VPL_VIEW_CAPABILITY','mod/vpl:view');
define('VPL_SUBMIT_CAPABILITY','mod/vpl:submit');
define('VPL_SIMILARITY_CAPABILITY','mod/vpl:similarity');
define('VPL_SETJAILS_CAPABILITY','mod/vpl:setjails');
define('VPL_MANAGE_CAPABILITY','mod/vpl:manage');

require_once dirname(__FILE__).'/vpl.class.php';
/**
 * Open/create a file and its dir
 *
 * @param $filename path to file
 * @return file descriptor
 **/
function vpl_fopen($filename){
	if(!file_exists($filename)){ // exists file?
		$dir = dirname($filename);
		if(!file_exists($dir)){ // create dir?
			mkdir($dir, 0700,true);
		}
	}
	$fp= fopen($filename,'wb+');
	return $fp;
}

/**
 * Recursively delete a directory
 *
 * @return bool All delete
 **/
function vpl_delete_dir($dirname){
	$ret = false;
	if(file_exists($dirname)){
		$ret = true;
		if(is_dir($dirname)){
			$dd = opendir($dirname);
			if(!$dd){
				return false;
			}
			$list = array();
			while($name=readdir($dd)){
				if($name != '.' && $name != '..'){
					$list[]=$name;
				}
			}
			closedir($dd);
			$ret = true;
			foreach ($list as $name) {
				$ret = vpl_delete_dir($dirname.'/'.$name) and $ret;
			}
			$ret = rmdir($dirname)and $ret;
		}
		else{
			$ret = unlink($dirname);
		}
	}
	return $ret;
}

/**
 * get parsed lines of a file
 *
 * @param $filename string
 * @return array of lines of the file
 **/
function vpl_read_list_from_file($filename){
	$ret = array();
	if(file_exists($filename)){
		$data = file_get_contents($filename);
		if($data>''){
			$nl = vpl_detect_newline($data);
			$ret = explode($nl,$data);
		}
	}
	return $ret;
}

/**
 * Save an array in a file
 * @param $filename string
 **/
function vpl_write_list_to_file($filename, $list){
	$data='';
	foreach ($list as $info) {
		if($info > ''){
			if($data > ''){
				$data .= "\n";
			}
			$data .= $info;
		}
	}
	$fp = vpl_fopen($filename);
	fwrite($fp,$data);
	fclose($fp);
}

/**
 * Get lang code
 * @parm $bash_adapt true adapt lang to bash LANG (default false)
 * @return string
 */
function vpl_get_lang($bash_adapt=false){
	global $SESSION,$USER,$CFG;
	if(isset($SESSION->lang)){
		$lang=$SESSION->lang;
	}elseif(isset($USER->lang)){
		$lang=$USER->lang;
	}else{
		$lang=$CFG->lang;
	}
	if($bash_adapt){
		$parts=explode('_',$lang);
		switch(count($parts)){
			case 2:
				if($parts[1]=='utf8'){
					$lang=$parts[0].'.UTF-8';
				}else{
					$lang=$parts[0].'_'.strtoupper($parts[1]);
				}
				break;
			case 3:
				if($parts[2]=='utf8'){
					$lang=$parts[0].'_'.strtoupper($parts[1]).'.UTF-8';
				}else{
					$lang=$parts[0].'_'.strtoupper($parts[1]).'.'.$parts[2];
				}
				break;
			default: //Do nothing
		}
	}
	return $lang;
}

/**
 * generate URL to page with params
 * @param $page string page from wwwroot
 * @param $var1 string var1 name optional
 * @param $value1 string value of var1 optional
 * @param $var2 string var2 name optional
 * @param $value2 string value of var2 optional
 * @param ...
 **/
function vpl_abs_href(){
	global $CFG;
	$parms = func_get_args();
	$l=count($parms);
	$href = $CFG->wwwroot.$parms[0];
	for( $p=1;$p<$l-1; $p+=2 ) {
		$href .= ($p>1?'&amp;':'?')
		.urlencode($parms[$p]).'='
		.urlencode($parms[$p+1]);
	}
	return $href;
}

/**
 * generate URL to page with params
 * @param $page string page from wwwroot/mod/vpl/
 * @param $var1 string var1 name optional
 * @param $value1 string value of var1 optional
 * @param $var2 string var2 name optional
 * @param $value2 string value of var2 optional
 * @param ...
 **/
function vpl_mod_href(){
	global $CFG;
	$parms = func_get_args();
	$l=count($parms);
	$href = $CFG->wwwroot.'/mod/vpl/'.$parms[0];
	for( $p=1;$p<$l-1; $p+=2 ) {
		$href .= ($p>1?'&amp;':'?')
		.urlencode($parms[$p]).'='
		.urlencode($parms[$p+1]);
	}
	return $href;
}

/**
 * generate URL relative page with params
 * @param $page string page relative
 * @param $var1 string var1 name optional
 * @param $value1 string value of var1 optional
 * @param $var2 string var2 name optional
 * @param $value2 string value of var2 optional
 * @param ...
 **/
function vpl_rel_url(){
	$parms = func_get_args();
	$l=count($parms);
	$url = $parms[0];
	for( $p=1;$p<$l-1; $p+=2 ) {
		$url .= ($p>1?'&amp;':'?')
		.urlencode($parms[$p]).'='
		.urlencode($parms[$p+1]);
	}
	return $url;
}
/**
 * Add a parm to a url
 * @param $url string
 * @param $parm string name
 * @param $value string value of parm
 **/
function vpl_url_add_param($url,$parm,$value){
	if(strpos($url,'?')){
		return $url . '&amp;'.urlencode($parm).'='.urlencode($value);
	}else{
		return $url . '?'.urlencode($parm).'='.urlencode($value);
	}
}

/**
 * Print a message and redirect
 * @param $message string to be print
 * @param $link URL to redirect to
 * @param $wait int time to wait in seconds
 * @return void
 */
function vpl_redirect($message,$link,$wait=4){
	print_box('<div style="text-align:center">'.$message.'</div>');
	redirect($link,'',$wait);
}

/**
 * Inmediate redirect
 * @param $link URL to redirect to
 * @return void
 */
function vpl_inmediate_redirect($url){
	vpl_redirect('',$url,0);
}

function vpl_include_jsfile($file,$defer=false){
	$ret='<script type="text/javascript"';
	if($defer){
		$ret.=' defer="defer"';
	}
	$url = vpl_mod_href('jscript/'.$file);
	$ret .=' src="'.$url.'"></script>';
	return $ret;
}

function vpl_include_js($jscript,$defer=false){
	$ret='<script type="text/javascript"';
	if($defer){
		$ret.=' defer="defer">';
	}else{
		$ret.='>';
	}
	$ret .="\n//<![CDATA[\n";
	$ret .= $jscript;
	$ret .="\n//]]>\n</script>\n";
	return $ret;
}

/**
 * Popup message box to show text
 * @param $text text to show. It use s() to sanitize text
 * @param $print
 * @return void
 */
function vpl_js_alert($text,$print=true){
	$aux = addslashes($text); //Sanitize text
	$aux = str_replace("\n","\\n",$aux); //Add \n to show multiline text
	$aux = str_replace("\r","",$aux); //Remove \r
	$ret=vpl_include_js('alert("'.$aux.'");');
	if($print){
		echo $ret;
		@ob_flush();
		flush();
	}else{
		return $ret;
	}
}

function vpl_get_select_time($limit=null){
	$minute=60;
	if($limit === null){ //Default value
		$limit = 35*$minute;
	}
	$ret = array(0 => get_string('select'));
	if($limit <= 0){
		return $ret;
	}
	$value=1;
	if($limit < $value){
		$value=$limit;
	}
	while($value <= $limit){
		if($value < $minute){
			$ret[$value] = get_string('numseconds','',$value);
		}else{
			$num = (int)($value / $minute);
			$ret[$num*$minute] = get_string('numminutes','',$num);
			$value = $num*$minute;
		}
		$value *=2;
	}
	return $ret;
}

/**
 * Return the post_max_size PHP config option in bytes
 * @return int max size in bytes
 */
function vpl_get_max_post_size(){
	$maxs = trim(ini_get('post_max_size'));
	$len = strlen($maxs);
	$last = strtolower($maxs[$len-1]);
	$max = (int) substr($maxs,0,$len-1);
	if($last == 'k'){
		$max *= 1024;
	}elseif ($last == 'm'){
		$max *= 1024*1024;
	}elseif ($last == 'g'){
		$max *= 1024*1024*1000;
	}
	return $max;
}

/**
 * Convert a size in byte to string in Kb, Mb, Gb and Tb
 * @param $size int size in bytes
 * @return string
 */
function vpl_conv_size_to_string($size){
	static $Kbyte=1024;
	static $Mbyte=1048576;
	static $Gbyte=1048576000;
	static $Tbyte=1048576000000;
	if($size < $Mbyte){
		$num = $size / $Kbyte;
		return sprintf('%4d KB',$num);
	}elseif($size < $Gbyte){
		$num = $size / $Mbyte;
		return sprintf('%4d MB',$num);
	}elseif($size < $Tbyte){
		$num = $size / $Gbyte;
		return sprintf('%4d GB',$num);
	}else{
		$num = $size / $Tbyte;
		return sprintf('%4d TB',$num);
	}
}

/**
 * Return un array with the format [size in bytes]=> size in text
 * The first element is [0] => select
 * @param $limit the limit of values generates
 * @return array
 */
function vpl_get_select_sizes($limit=PHP_INT_MAX){
	$limit = (int) $limit;
	if($limit < 0){
		$limit = PHP_INT_MAX;
	}
	$ret = array(0 => get_string('select'));
	$value=4*1024;
	while($value <= $limit && $value >0){ //Avoid int overflow
		$ret[$value] = vpl_conv_size_to_string($value);
		$pre=$value;
		$value *=2;
		$value = (int) $value;
		if($pre >= $value){ //Check for loop end
			break;
		}
	}
	if($value>0 && $limit > ($value*0.75)){ //Show limit value
		$ret[$limit] = vpl_conv_size_to_string($limit);
	}
	return $ret;
}

/**
 * @param string $data
 * @return string newline separator "\r\n", "\n", "\r"
 */
function vpl_detect_newline(&$data){
	//Detect text newline chars
	if(strrpos($data,"\r\n") !== false){
		return "\r\n"; //Windows
	}else if(strrpos($data,"\n") !== false){
		return "\n"; //UNIX
	}else if(strrpos($data,"\r") !== false){
		return "\r"; //Mac
	}else{
		return "\n"; //Default Unix
	}
}

function vpl_display_grade($grade,$scaleid) {
	global $CFG;
	require_once $CFG->libdir.'/gradelib.php';
	static $scalegrades = array();   //Cache scales
	if ($scaleid >= 0) {    //Normal scale
		if ($grade == -1) {
			return '-';
		} else {
			return $grade.' / '.$scaleid;
		}
	} else {
		$scaleid = -$scaleid;
		if (empty($scalegrades[$scaleid])) {
			if ($scale = get_record('scale', 'id', $scaleid)) {
				$scalegrades[$scaleid] = make_menu_from_list($scale->scale);
			} else {
				return '-';
			}
		}
		if (isset($scalegrades[$scaleid][$grade])) {
			return $scalegrades[$scaleid][$grade];
		}
		return '-';
	}
}

?>